==> lockjaw-0.2.1.crate: lockjaw-0.2.1/userguide/src/setup.md <==
# Project Setup

## Cargo

Add lockjaw to the `[dependencies]` and `[build_dependencies]` section of your `Cargo.toml`:

```toml
{{ #include ../projects/setup/Cargo.toml:dep}}
```

The proc_macro and runtime library are packaged into the same crate, so this is the only target you
need.

## Build script

Lockjaw also needs some environment setup, and requires a
[build script](https://doc.rust-lang.org/cargo/reference/build-scripts.html). Add `build.rs` next to
`Cargo.toml`, and
call [`lockjaw::build_script()`](https://docs.rs/lockjaw/0.2.0/lockjaw/fn.build_script.html)
in `main()` inside it:

```rust,no_run,noplayground
// https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/build.rs
{{#include ../projects/setup/build.rs:main}}
```

Lockjaw will ask you to do this if this step is missing.

## Prologue macro

Before using any Lockjaw attribute macros, the
lockjaw [`prologue!()`](https://docs.rs/lockjaw/0.2.0/lockjaw/macro.prologue.html) macro must be
called:

```rust,no_run,noplayground
// https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/src/main.rs
{{#include ../projects/setup/src/main.rs:prologue}}
```

The file path of the current source file should be passed to the prologue.

Lockjaw need to perform some static analysis on the current source file to understand type names,
which is an integral part of a dependency injection framework. If a type is encountered, what is its
fully qualified name? Is it something imported with a `use` declaration? or if it is a locally
declared, what is the `mod` path to the local scope?

Lockjaw also generates a test to verify the source path is correct. However passing a wrong path
often result in weird type resolution errors at compile time. Double-check the source path if there
are type resolution issues.

## Epilogue macro

You also must call
the [`lockjaw::epilogue!()`](https://docs.rs/lockjaw/0.2.0/lockjaw/macro.epilogue.html) macro in the
root of your crate (`lib.rs` or
`main.rs`) after all other uses of lockjaw, preferably at the end of the file.

```rust,no_run,noplayground
// https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/src/main.rs
{{#include ../projects/setup/src/main.rs:epilogue}}
```

[Source](https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/) of this chapter

==> lockjaw-0.2.2.crate: lockjaw-0.2.2/userguide/src/setup.md <==
# Project Setup

## Cargo

Add lockjaw to the `[dependencies]` and `[build_dependencies]` section of your `Cargo.toml`:

```toml
{{ #include ../projects/setup/Cargo.toml:dep}}
```

The proc_macro and runtime library are packaged into the same crate, so this is the only target you
need. While the `proc_macro` library is heavy, Rust should be able to optimize them away in the
resulting binary. The runtime is pretty light, and the generated code is *supposed* to be zero cost
abstraction.

## Build script

Lockjaw also needs some environment setup, and requires a
[build script](https://doc.rust-lang.org/cargo/reference/build-scripts.html). Add `build.rs` next to
`Cargo.toml`, and
call [`lockjaw::build_script()`](https://docs.rs/lockjaw/latest/lockjaw/fn.build_script.html)
in `main()` inside it:

```rust,no_run,noplayground
// https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/build.rs
{{#include ../projects/setup/build.rs:main}}
```

Lockjaw will ask you to do this if this step is missing.

## Prologue macro

Before using any Lockjaw attribute macros, the
lockjaw [`prologue!()`](https://docs.rs/lockjaw/latest/lockjaw/macro.prologue.html) macro must be
called:

```rust,no_run,noplayground
// https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/src/main.rs
{{#include ../projects/setup/src/main.rs:prologue}}
```

The file path of the current source file should be passed to the prologue.

Lockjaw need to perform some static analysis on the current source file to understand type names,
which is an integral part of a dependency injection framework. If a type is encountered, what is its
fully qualified name? Is it something imported with a `use` declaration? or if it is a locally
declared, what is the `mod` path to the local scope?

Lockjaw also generates a test to verify the source path is correct. However passing a wrong path
often result in weird type resolution errors at compile time. Double-check the source path if there
are type resolution issues.

## Epilogue macro

You also must call
the [`lockjaw::epilogue!()`](https://docs.rs/lockjaw/latest/lockjaw/macro.epilogue.html) macro in the
root of your crate (`lib.rs` or
`main.rs`) after all other uses of lockjaw, preferably at the end of the file.

```rust,no_run,noplayground
// https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/src/main.rs
{{#include ../projects/setup/src/main.rs:epilogue}}
```

[Source](https://github.com/azureblaze/lockjaw/tree/main/userguide/projects/setup/) of this chapter
